  var scene = new THREE.Scene();

    // SETUP RENDERER

  var renderer = new THREE.WebGLRenderer();
  renderer.setSize( window.innerWidth, window.innerHeight );
  renderer.setClearColor( 0xffffff );                // white background colour
  document.body.appendChild( renderer.domElement );  // add to document canvas 

   // SETUP CAMERA

  var aspect = window.innerWidth/window.innerHeight;
  camera = new THREE.PerspectiveCamera( 30, aspect, 0.1, 10000);   // view angle, aspect ratio, near, far
  camera.position.set(10,15,40);
  camera.lookAt(scene.position);	
  scene.add(camera);

    // DEFINE UNIT CUBE -- to be reused several times

  var unitCubeGeometry = new THREE.BoxGeometry( 1,1,1 );   

    // SCENE AXES:    (x,y,z) drawn in (red,greeen,blue)

  var redMaterial = new THREE.MeshBasicMaterial( { color: 0xff0000 } );
  var greenMaterial = new THREE.MeshBasicMaterial( { color: 0x00ff00 } );
  var blueMaterial  = new THREE.MeshBasicMaterial( { color: 0x0000ff } );
  var yellowMaterial  = new THREE.MeshBasicMaterial( { color: 0xffff00 } );
  var xAxis = new THREE.Mesh( unitCubeGeometry, redMaterial );
  var yAxis = new THREE.Mesh( unitCubeGeometry, greenMaterial );
  var zAxis = new THREE.Mesh( unitCubeGeometry, blueMaterial );
  var axisWidth = 0.2;
  var axisLength = 10;
  scene.add( xAxis );   xAxis.scale.set(axisLength,axisWidth,axisWidth); xAxis.position.set(0.5*axisLength,0,0);
  scene.add( yAxis );   yAxis.scale.set(axisWidth,axisLength,axisWidth); yAxis.position.set(0,0.5*axisLength,0);
  scene.add( zAxis );   zAxis.scale.set(axisWidth,axisWidth,axisLength); zAxis.position.set(0,0,0.5*axisLength);
  var originBox = new THREE.Mesh( unitCubeGeometry, yellowMaterial );
  scene.add( originBox );
  xAxis.parent = yAxis.parent = zAxis.parent = originBox;
  
    // FLOOR WITH CHECKERBOARD 

  var floorTexture = new THREE.ImageUtils.loadTexture( 'images/checkerboard.jpg' );
  floorTexture.wrapS = floorTexture.wrapT = THREE.RepeatWrapping; 
  floorTexture.repeat.set( 4, 4 );
  var floorMaterial = new THREE.MeshBasicMaterial( { map: floorTexture, side: THREE.DoubleSide } );
  var floorGeometry = new THREE.PlaneGeometry(30, 30);
  var floor = new THREE.Mesh(floorGeometry, floorMaterial);
  floor.position.y = -0.1;
  floor.rotation.x = Math.PI / 2;
  scene.add(floor);
  floor.parent = originBox;

      // LIGHTS:  needed for phong illumination model
      // The following will be used by (1) three.js; and (2) your own shaders, passed via uniforms

  lightColor = new THREE.Color(1,1,1);
  ambientColor = new THREE.Color(1,1,1);
  lightPosition = new THREE.Vector3(-70,100,70);

/////////////////////////// THREE.JS ILLUMINATION ////////////////////////////

    // LIGHT SOURCES

  var light = new THREE.PointLight(lightColor.getHex());
  light.position.set(lightPosition.x, lightPosition.y, lightPosition.z);
  scene.add(light);
  var ambientLight = new THREE.AmbientLight(ambientColor.getHex());
  scene.add(ambientLight);

    // DEFAULT PHONG MATERIAL  (used by the car + statue)

  var defaultPhongMaterial = new THREE.MeshPhongMaterial( { 
       ambient: 0x004000, color: 0x008000, specular: 0x404040, shininess: 40.0});

    // EXAMPLE SPHERE

  var sphereGeometry = new THREE.SphereGeometry( 2, 32, 32 );
  var examplePhongMaterial = new THREE.MeshPhongMaterial( { 
       ambient: 0x004000, color: 0x008000, specular: 0x404040, shininess: 40.0});
  var exampleSphere = new THREE.Mesh( sphereGeometry, examplePhongMaterial );
  scene.add( exampleSphere );            // add to scene
  exampleSphere.position.set(13,1,0);    // reposition it where we want
  exampleSphere.parent = originBox;      // make it a child of the rotating checkerboard

    // AMBIENT SPHERE:  place at (-6,1,10), render with ambient reflectance of (0.25,0.25,0.25)


    // DIFFUSE SPHERE:  place at (-2,1,10), render with diffuse reflectance of (0.5,0.5,0.5)


    // SPECULAR:  place at (2,1,10), render with specular reflectance of (0.125, 0.125, 0.125), shininess=40


    // ALL:  place at (6,1,10), render with ambient + diffuse + specular as used above


/////////////////////////// MY SHADERS ////////////////////////////

  kAmbient = new THREE.Color(0x404040);    // ambient reflectance
  kDiffuse = new THREE.Color(0x808080);    // diffuse reflectance
  kSpecular = new THREE.Color(0x808080);   // specular reflectance (purposely 4x larger)
  k0 = new THREE.Color(0x000000);          // use when we want to assign zero reflectance
  shininess = 10.0;                        // Phong shininess (purposely 4x smaller)

    // AMBIENT

  var mySphere1Material = new THREE.ShaderMaterial({
    uniforms: {
      lightColor: {type: "c", value: lightColor },
      ambientColor: {type: "c", value: ambientColor },
      lightPosition: {type: "v3", value: lightPosition },
      nSpec: {type: "f", value: shininess },
      ka: {type: "c", value: kAmbient },
      kd: {type: "c", value: k0 },
      ks: {type: "c", value: k0 }
    },
    vertexShader: document.getElementById('vertexShader').text,
    fragmentShader: document.getElementById('myPhong').text
  });
  var mySphere1Geometry = new THREE.SphereGeometry( 2, 32, 32 );
  var mySphere1 = new THREE.Mesh( mySphere1Geometry, mySphere1Material );
  scene.add( mySphere1 );
  mySphere1.position.set(-6,1,5);
  mySphere1.parent = originBox;

    // DIFFUSE

  var mySphere2Material = new THREE.ShaderMaterial({
    uniforms: {
      lightColor: {type: "c", value: lightColor },
      ambientColor: {type: "c", value: ambientColor },
      lightPosition: {type: "v3", value: lightPosition },
      nSpec: {type: "f", value: shininess },
      ka: {type: "c", value: k0 },
      kd: {type: "c", value: kDiffuse },
      ks: {type: "c", value: k0 }
    },
    vertexShader: document.getElementById('vertexShader').text,
    fragmentShader: document.getElementById('myPhong').text
  });
  var mySphere2Geometry = new THREE.SphereGeometry( 2, 32, 32 );
  var mySphere2 = new THREE.Mesh( mySphere2Geometry, mySphere2Material );
  scene.add( mySphere2 );
  mySphere2.position.set(-2,1,5);
  mySphere2.parent = originBox;

    // SPECULAR

  var mySphere3Material = new THREE.ShaderMaterial({
    uniforms: {
      lightColor: {type: "c", value: lightColor },
      ambientColor: {type: "c", value: ambientColor },
      lightPosition: {type: "v3", value: lightPosition },
      nSpec: {type: "f", value: shininess },
      ka: {type: "c", value: k0 },
      kd: {type: "c", value: k0 },
      ks: {type: "c", value: kSpecular }
    },
    vertexShader: document.getElementById('vertexShader').text,
    fragmentShader: document.getElementById('myPhong').text
  });
  var mySphere3Geometry = new THREE.SphereGeometry( 2, 32, 32 );
  var mySphere3 = new THREE.Mesh( mySphere3Geometry, mySphere3Material );
  scene.add( mySphere3 );
  mySphere3.position.set(2,1,5);
  mySphere3.parent = originBox;

    // ALL

  var mySphere4Material = new THREE.ShaderMaterial({
    uniforms: {
      lightColor: {type: "c", value: lightColor },
      ambientColor: {type: "c", value: ambientColor },
      lightPosition: {type: "v3", value: lightPosition },
      nSpec: {type: "f", value: shininess },
      ka: {type: "c", value: kAmbient },
      kd: {type: "c", value: kDiffuse },
      ks: {type: "c", value: kSpecular }
    },
    vertexShader: document.getElementById('vertexShader').text,
    fragmentShader: document.getElementById('myPhong').text
  });
  var mySphere4Geometry = new THREE.SphereGeometry( 2, 32, 32 );
  var mySphere4 = new THREE.Mesh( mySphere4Geometry, mySphere4Material );
  scene.add( mySphere4 );
  mySphere4.position.set(6,1,5);
  mySphere4.parent = originBox;

    // GREEN CUBE: phong illumination model, evaluated per-pixel, but with specular term set to zero

  var greenPhongMaterial = new THREE.MeshPhongMaterial( 
    {  ambient: 0x000f00, color: 0x008000, specular: 0x008000, shininess: 30, shading: THREE.SmoothShading });
  var greenCubeGeometry = new THREE.BoxGeometry( 1,1,1 );   
  var greenCube = new THREE.Mesh( greenCubeGeometry, greenPhongMaterial );
  scene.add( greenCube );
  greenCube.scale.set(2,2,2);
  greenCube.position.set(7,1,0);
  greenCube.parent = originBox;
  
    // RED CUBE with basic shading, i.e., single colour

  var redMaterial = new THREE.MeshBasicMaterial( { color: 0xff0000 } );
  var redCube = new THREE.Mesh( unitCubeGeometry, redMaterial );
  scene.add( redCube );
  redCube.scale.set(2,2,2);
  redCube.position.set(4,1,0);
  redCube.parent = originBox;

    // MULTI-COLORED CUBE

  var cubeMaterialArray = [];
  cubeMaterialArray.push( new THREE.MeshBasicMaterial( { color: 0xff3333 } ) ); // red
  cubeMaterialArray.push( new THREE.MeshBasicMaterial( { color: 0xff8800 } ) ); // orange
  cubeMaterialArray.push( new THREE.MeshBasicMaterial( { color: 0xffff33 } ) ); // yellow
  cubeMaterialArray.push( new THREE.MeshBasicMaterial( { color: 0x33ff33 } ) ); // green
  cubeMaterialArray.push( new THREE.MeshBasicMaterial( { color: 0x3333ff } ) ); // blue
  cubeMaterialArray.push( new THREE.MeshBasicMaterial( { color: 0x8833ff } ) ); // purple
  var cubeMaterials = new THREE.MeshFaceMaterial( cubeMaterialArray );
  var cubeGeometry = new THREE.BoxGeometry( 2, 2, 2);             // cube dimensions
  colourCube = new THREE.Mesh( cubeGeometry, cubeMaterials );
  colourCube.position.set(10,1,0);
  scene.add( colourCube );		
  colourCube.parent = originBox;

    // TEXTURED CUBE:  the crate

  var crateTexture = new THREE.ImageUtils.loadTexture( 'images/crate.gif' );
  var crateMaterial = new THREE.MeshBasicMaterial( { map: crateTexture } );
  var crateGeometry = new THREE.BoxGeometry( 1,1,1 );   
  var crate = new THREE.Mesh( crateGeometry, crateMaterial );
  crate.position.set(-5,2,-5);
  crate.scale.set(4,4,4);
  scene.add( crate );
  crate.parent = originBox;

     // TEAPOT

  var teapot;
  new THREE.JSONLoader().load('js/teapot.js', function(geometry) { 
//    var teapotMaterial = new THREE.MeshBasicMaterial({ color: 0xff0000, wireframe: true } );
//    var teapot.material = defaultPhongMaterial;
    var teapotMaterial	= new THREE.MeshNormalMaterial(); 
    teapot	= new THREE.Mesh( geometry, teapotMaterial ); 
    scene.add( teapot ); 
    teapot.parent = originBox;
    teapot.position.set(0,0,-10);
  }); 

   // TORUS KNOT

  var myShaderMaterial = new THREE.ShaderMaterial({
    uniforms: {
      lightColor: {type: "c", value: lightColor },
      ambientColor: {type: "c", value: ambientColor },
      lightPosition: {type: "v3", value: lightPosition },
      nSpec: {type: "f", value: shininess },
      ka: {type: "c", value: kAmbient },
      kd: {type: "c", value: kDiffuse },
      ks: {type: "c", value: kSpecular }
    },
    vertexShader: document.getElementById('vertexShader').text,
    fragmentShader: document.getElementById('myPhong').text
  });
  var torusKnotGeometry = new THREE.TorusKnotGeometry( 10, 3, 100, 16 );
  var torusKnot = new THREE.Mesh( torusKnotGeometry, myShaderMaterial );
  scene.add( torusKnot );
  torusKnot.scale.set(0.15,0.15,0.15);
  torusKnot.position.set(10,1,-9);
  torusKnot.parent = originBox;

     // STATUE + CAR:  meshes laoded from OBJ files

   loadOBJ( 'obj/minicooper.obj',0.05, -10,0,0,-1.57,0,0);
   loadOBJ( 'obj/male02.obj', 0.05, 0,0,1,0,0,0);
//   loadOBJ( '/Users/van/threejs-all/three.js/examples/obj/male02/minicooper.obj',0.05, -10,0,0,-1.57,0,0);
//   loadOBJ( '/Users/van/threejs-all/three.js/examples/obj/male02/male02.obj', 0.05, 0,0,1,0,0,0);

  

    // SETUP RENDER CALL-BACK

  var render = function () {
    requestAnimationFrame( render );
    originBox.rotation.y += 0.004;
    renderer.render(scene, camera);
  };
  render();
